// This file is currently unlocked (change this line if you lock the file)
//
// Source code version is 2.5.5
//                        (increment this number EVERY time you make a change)
//
// History: (add to this EVERY time you make a change) 
// (insert details of your change immediately below this line)
//
// 2.5.5        09 Jul 1997     Jill Baker
//      Bug fix to deflate(). Didn't work before. Does now.
//
// 2.5.4        14 May 1997     Jill Baker
//      Changed so that data stored internally as compressed. Added constructors
//      ( byte[] ) and ( byte[], int ). Added methods getRawData() and
//      getPacketData(). Deprecated getData(). Added public static methods
//      inflate() and deflate(). Added tests for Java 1.0 compatibility.
//      setAlgorithm() is now private.
//
// 2.5.3        10 Apr 1997     Jill Baker
//      Added getType() method
//
// 2.5.2        03 Apr 1997     Jill Baker
//      setAlgorithm() now throws a FormatException with invalid algorithms.
//
// 2.5.1        01 Apr 1997     Jill Baker
//      This file has more or less been completely rewritten from scratch.
//      It would be fair to say that 2.5.0 was only really a skeleton.
//
// 2.5.0        24 Mar 1997     Original Author not stated
//      Original version
//
// If this file is unlocked, or locked by someone other than you, then you
// may download this file for incorporation into your build, however any
// changes you make to your copy of this file will be considered volatile,
// and will not be incorporated into the master build which is maintained
// at this web site.
//
// If you wish to make permanent changes to this file, you must wait until
// it becomes unlocked (if it is not unlocked already), and then lock it
// yourself. Whenever the file is locked by you, you may upload new master
// versions to the master site. Make sure you update the history information
// above. When your changes are complete, you should relinquish the lock to
// make it available for other developers.

/*
 * Copyright (C) 1995, 1996 Systemics Ltd (http://www.systemics.com/)
 * All rights reserved.
 *
 */

package cryptix.pgp;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInput;
import java.io.DataOutput;
import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import cryptix.util.Streamable;

/**
 * Implements PGP compressed data packet
 * @author Jill Baker
 */
public final class CompressedData extends Packet implements Algorithm
{
    private int algorithm;
    private byte[] compressedData;

    /**
     * Create a CompressedData object compressing binary data using the ZIP compression algorithm.
     * @param rawdata any binary data to be compressed.
     */
    public CompressedData( byte[] rawData ) throws IOException
    {
        algorithm = ZIP;
        compressedData = deflate( rawData );
    }

    /**
     * Create a CompressedData object compressing binary data using any compression algorithm.
     * @param rawdata any binary data to be compressed.
     * @param algorithm algorithm to use for compression (use one of the constants in cryptix.pgp.Algorithm).
     */
    public CompressedData( byte[] rawData, int algorithm )
    throws IOException
    {
        setAlgorithm( algorithm );
        compressedData = deflate( rawData );
    }

    /**
     * Create a CompressedData object compressing PGP packet data using the ZIP compression algorithm.
     * @param packetByteArray data to be compressed.
     */
    public CompressedData( PacketByteArray packetByteArray ) throws IOException
    {
        algorithm = ZIP;
        compressedData = deflate( packetByteArray.toByteArray() );
    }

    /**
     * Create a CompressedData object compressing PGP packet data using any compression algorithm.
     * @param packetByteArray data to be compressed.
     * @param algorithm algorithm to use for compression (use one of the constants in cryptix.pgp.Algorithm).
     */
    public CompressedData( PacketByteArray packetByteArray, int algorithm )
    throws IOException
    {
        setAlgorithm( algorithm );
        compressedData = deflate( packetByteArray.toByteArray() );
    }

    /**
     * Create a CompressedData object from an input stream
     * @param in DataInput input stream
     * @param length number of bytes to input
     */
    public CompressedData( DataInput in, int length )
		throws IOException
	{
        super( in, length );
	}

    /**
     * Throw an exception if algorithm byte is not valid
     * @param algorithm value to test
     */
    public static void ensureValidAlgorithm( int algorithm ) throws FormatException
    {
        switch (algorithm)
        {
            case NONE:
            case ZIP:
            case GZIP:
                break;

            default:
                throw new FormatException(
                "Unknown compression algorithm (" + algorithm + ")" );
        }
    }

    /**
     * @return the compression algorithm used by this object
     */
    public int getAlgorithm()
    {
        return algorithm;
    }

    private void setAlgorithm( int algorithm ) throws FormatException
    {
        ensureValidAlgorithm( algorithm );
        this.algorithm = algorithm;
    }

    /**
     * @deprecated
     */
    public PacketByteArray getData() throws FormatException, IOException
    {
        return getPacketData();
    }

    /**
     * @return decompressed data
     */
    public byte[] getRawData() throws IOException
    {
        return inflate( compressedData );
    }

    /**
     * @return decompressed data
     */
    public PacketByteArray getPacketData() throws FormatException, IOException
    {
        return new PacketByteArray( inflate( compressedData ));
    }

    public void read( DataInput in, int length )
    throws IOException
    {
        compressedData = null;

        setAlgorithm( in.readByte() );
        length--;
        if (length >= 0)
        {
            compressedData = new byte[length-1];
            in.readFully( compressedData );
        }
        else
        {
            InputStream is = (InputStream) in;
            BufferedInputStream sin = new BufferedInputStream( is );
            ByteArrayOutputStream sout = new ByteArrayOutputStream();

            for (int b=sin.read(); b!=-1; b=sin.read()) sout.write( b );

            compressedData = sout.toByteArray();
        }
	}

	public int
	write( DataOutput out ) throws IOException
	{
        out.writeByte( (byte)algorithm );
        out.write( compressedData );
        return 1 + compressedData.length;
    }

    /**
     * Inflate (decompress) an arbitrary byte array
     * @param compressedData data to inflate
     * @param algorithm algorithm to use
     * @return uncompressed data
     */
    public static byte[] inflate( byte[] compressedData, int algorithm )
    throws IOException, FormatException
    {
        ensureValidAlgorithm( algorithm );

        ByteArrayInputStream bs = new ByteArrayInputStream( compressedData );
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        InputStream in;
        switch (algorithm)
        {
            case ZIP:
            {
                java.util.zip.Inflater inf
                   = new java.util.zip.Inflater( true );
                in = new java.util.zip.InflaterInputStream( bs, inf );
                break;
            }
            default:
            {
                return compressedData;
            }
        }
        for (int c=in.read(); c != -1; c=in.read()) out.write( c );
        return out.toByteArray();
    }

    /**
     * Deflate (compress) an arbitrary byte array
     * @param uncompressedData data to deflate
     * @param algorithm algorithm to use
     * @return compressed data
     */
    public static byte[] deflate( byte[] uncompressedData, int algorithm )
    throws IOException, FormatException
    {
        ensureValidAlgorithm( algorithm );

        ByteArrayInputStream in = new ByteArrayInputStream( uncompressedData );
        ByteArrayOutputStream bout = new ByteArrayOutputStream();
        switch (algorithm)
        {
            case ZIP:
            {
                java.util.zip.Deflater def = new java.util.zip.Deflater( java.util.zip.Deflater.DEFAULT_COMPRESSION, true );
                java.util.zip.DeflaterOutputStream out = new java.util.zip.DeflaterOutputStream( bout, def );
                for (int c=in.read(); c != -1; c=in.read()) out.write( c );
                out.finish();
                return bout.toByteArray();
            }
            default:
            {
                return uncompressedData;
            }
        }
        
    }

    private byte[] inflate( byte[] compressedData ) throws IOException, FormatException
    {
        return inflate( compressedData, algorithm );
    }

    private byte[] deflate( byte[] uncompressedData ) throws IOException
    {
        return deflate( uncompressedData, algorithm );
    }

    /**
     * @return PGP code for compressed data packet
     */
    public int getType()
    {
        return Packet.COMPRESSED;
    }
}
