// This file is currently unlocked (change this line if you lock the file)
//
// Source code version is 2.5.1
//                        (increment this number EVERY time you make a change)
//
// History: (add to this EVERY time you make a change) 
// (insert details of your change immediately below this line)
//
// 2.5.2        03 May 1997     Ian Brown
//      added doc comments
//
// 2.5.1        15 Apr 1997     Jill Baker
//      imports changed to new namespace
//
// 2.5.0        24 Mar 1997     Original Author not stated
//      Original version
//
// If this file is unlocked, or locked by someone other than you, then you
// may download this file for incorporation into your build, however any
// changes you make to your copy of this file will be considered volatile,
// and will not be incorporated into the master build which is maintained
// at this web site.
//
// If you wish to make permanent changes to this file, you must wait until
// it becomes unlocked (if it is not unlocked already), and then lock it
// yourself. Whenever the file is locked by you, you may upload new master
// versions to the master site. Make sure you update the history information
// above. When your changes are complete, you should relinquish the lock to
// make it available for other developers.

package cryptix.pgp;

import java.io.IOException;
import cryptix.security.MD5;
import cryptix.security.rsa.PublicKey;

/**
 * Abstract base class for PGP keyrings and keyservers.<p> 
 * <hr>
 * <P>
 * <bold>Copyright</bold> &#169 1995, 1996, 1997
 * <a href="http://www.systemics.com/">Systemics Ltd</a>
 * on behalf of the
 * <a href="http://www.systemics.com/docs/cryptix/">
 * Cryptix Development Team</a>.
 * All rights reserved.
 * @version 1.1, 29/11/1996
 * @author Ian Brown
 * @see systemics.pgp.PGPKeyRing systemics.pgp.KeyServer
 */
public abstract class KeyStore
{
	protected abstract PublicKey getKey( String username ) throws IOException;
	protected abstract PublicKey getKey( KeyID keyID ) throws IOException;

	/**
	 * Check the signature <code>sig</code> on <code>cert</code>
	 * and <code>username</code> using the keys in this keystore.
	 */
	public final boolean 
	checkSignature( Signature sig, Certificate cert, UserId name )
	throws IOException
	{
		return checkSignature( this, sig, cert, name );
	}
	
	/**
	 * Check the signature <code>sig</code> on <code>cert</code>
	 * and <code>username</code> using the keys in <code>store</code>.
	 */
	public final static boolean 
	checkSignature( KeyStore store, Signature sig, Certificate cert, UserId name )
	throws IOException
	{
		PublicKey key;
		if ( (key = store.getKey( sig.getKeyId() )) != null )
			return checkSignature( key, sig, cert, name );
		throw new IOException( "Key id " + sig.getKeyId() + " not found" );
	}
	
	/**
	 * Check the signature <code>sig</code> on <code>key</code>'s <code>cert</code>
	 * and <code>username</code> using the keys in <code>store</code>.
	 */
	public final static boolean 
	checkSignature( PublicKey key, Signature sig, Certificate cert, UserId name )
	throws IOException
	{
		MD5 md = new MD5();
		md.add( PacketFactory.save( cert ) );
		md.add( name.save() );
		return sig.check( key, md );
	}
}

