// This file is currently unlocked (change this line if you lock the file)
//
// Source code version is 2.5.5
//                        (increment this number EVERY time you make a change)
//
// History: (add to this EVERY time you make a change) 
// (insert details of your change immediately below this line)
//
// 2.5.5        13 Jun 1997     Jill Baker
//      replaced deprecated constructor String(byte[],0)
//      with non-deprecated factory method LegacyString.toString(byte[]).
//      Now checks that mode is valid.
//
// 2.5.4        12 Jun 1997     Jill Baker
//      Tweaked to ensure Java 1.0.x compatibility.
//
// 2.5.3        30 May 1997     Jill Baker
//      Added method isText(). Constructor (byte[], String, char) now
//      checks whether files claimed to be text files really are.
//
// 2.5.2        11 Apr 1997     Jill Baker
//      Added the method getType()
//
// 2.5.1        01 Apr 1997     Jill Baker
//      Added the method getBuffer() because I needed to get (public)
//      access to the buffer, without having to slow things down by copying.
//
//      QUESTION: Is the method data() neccessary?
//
// 2.5.0        24 Mar 1997     Original Author not stated
//      Original version
//
// If this file is unlocked, or locked by someone other than you, then you
// may download this file for incorporation into your build, however any
// changes you make to your copy of this file will be considered volatile,
// and will not be incorporated into the master build which is maintained
// at this web site.
//
// If you wish to make permanent changes to this file, you must wait until
// it becomes unlocked (if it is not unlocked already), and then lock it
// yourself. Whenever the file is locked by you, you may upload new master
// versions to the master site. Make sure you update the history information
// above. When your changes are complete, you should relinquish the lock to
// make it available for other developers.

/*
 * Copyright (C) 1995, 1996 Systemics Ltd (http://www.systemics.com/)
 * All rights reserved.
 *
 */

package cryptix.pgp;

import cryptix.mime.LegacyString;
import cryptix.util.Streamable;
import java.util.Date;
import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;

public final class LiteralData extends Packet
{
	private Date timestamp;
	private String filename;
	private char mode;
	private byte buffer[];

	public LiteralData( byte data[] )
	{
        try { init( data, "data", 'b' ); }
        catch (FormatException e) { /* Can never happen */ }
	}

	public LiteralData( byte data[], String filename, char mode ) throws FormatException
	{
        init( data, filename, mode );
	}

    private void init( byte data[], String filename, char mode ) throws FormatException
	{
		this.filename = filename.length() < 255 ? filename : filename.substring( 0, 255 );
		this.mode = mode;
		timestamp = new Date();
		buffer = data;

        // Ensure mode is valid
        switch (mode)
        {
            case 't':
            case 'b':
                break;
            default:
                throw new FormatException( "Invalid mode for LiteralData packet" );
        }

        // Change mode to 'b' if not a text file.
        if ((mode == 't') && (!isText(buffer))) mode = 'b';
	}


    public LiteralData( DataInput in, int length )
		throws IOException
	{
		super( in, length );
	}
public int getDefaultLengthSpecifier() { return 4; }

	public Date timestamp() { return timestamp; }
	public String filename() { return filename; }
	public char mode() { return mode; }
	public boolean modeIsText() { return mode == 't'; }
	public boolean modeIsBinary() { return mode == 'b'; }

    public byte[] getBuffer()
    {
        return buffer;
    }

	public byte[]
	data()
	{
		int len;
		byte tmp[] = new byte[len = buffer.length];
		System.arraycopy( buffer, 0, tmp, 0, len );
		return tmp;
	}

	public void 
	read( DataInput in, int length )
		throws IOException
	{
          
		if (length < 6)
			throw new FormatException("Length too short");

		mode = (char)in.readByte(); // mode
		if (mode != 'b' && mode != 't')
			throw new FormatException("Invalid mode");

		int nameLength = in.readByte() & 0xFF;
		if ( nameLength > 0 )
		{
			byte tmp[] = new byte[nameLength];
			in.readFully( tmp );
			filename = LegacyString.toString( tmp );
		}
		else
			filename = "";

		timestamp = new Date((in.readInt() & 0xFFFFFFFFL) * 1000L);

		buffer = new byte[length-nameLength-6];
		in.readFully(buffer);
	}

	public int
	write( DataOutput out )
		throws IOException
	{
		int nameLength = filename.length();
		out.writeByte( (byte)mode );
		out.writeByte( (byte)nameLength );
		out.writeBytes( filename );
		out.writeInt((int)(timestamp.getTime()/1000L));
		out.write( buffer );
		return 6 + nameLength + buffer.length;
	}

    public int getType()
    {
        return Packet.PLAINTEXT;
    }

    private boolean isText( byte[] buffer )
    {
        int len = buffer.length;
        if (len == 0) return false;
        for (int i=0; i<len; i++)
        {
            char c = (char)(buffer[i] & 0xFF);
            if ((c <= '\u001F') || (c >= '\u007F' && c <= '\u009F'))
            {
                switch (c)
                {
                    // The following control chars are allowed in text files:
                    case '\u0007':  // BEL
                    case '\u0008':  // BS
                    case '\u0009':  // HT
                    case '\n':      // LF
                    case '\u000B':  // VT
                    case '\u000C':  // FF
                    case '\r':      // CR
                    case '\u001A':  // EOF
                    case '\u001B':  // ESC
                    case '\u009B':  // CSI
                        continue;
                    // Anything else, isn't.
                    default:
                        return false;
                }
            }
        }
        return true;
    }
}
