// This file is currently unlocked (change this line if you lock the file)
//
// Source code version is 2.5.3
//                        (increment this number EVERY time you make a change)
//
// History: (add to this EVERY time you make a change) 
// (insert details of your change immediately below this line)
//
// 2.5.3        1 July 1997     Ian Brown
//      fixed to work with changes to KeyRing
//
// 2.5.2        03 May 1997     Ian Brown
//      added doc comments
//
// 2.5.1        15 Apr 1997     Jill Baker
//      imports changed to new namespace
//
// 2.5.0        24 Mar 1997     Original Author not stated
//      Original version
//
// If this file is unlocked, or locked by someone other than you, then you
// may download this file for incorporation into your build, however any
// changes you make to your copy of this file will be considered volatile,
// and will not be incorporated into the master build which is maintained
// at this web site.
//
// If you wish to make permanent changes to this file, you must wait until
// it becomes unlocked (if it is not unlocked already), and then lock it
// yourself. Whenever the file is locked by you, you may upload new master
// versions to the master site. Make sure you update the history information
// above. When your changes are complete, you should relinquish the lock to
// make it available for other developers.

package cryptix.pgp;

import java.io.InputStream;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.File;

import java.util.Vector;
import java.util.Enumeration;

import cryptix.security.rsa.PublicKey;
import cryptix.security.rsa.SecretKey;

import cryptix.util.EmailAddress;

/**
 * Represents a public keyring. Keys on it can be retrieved using the
 * <code>getKey()</code> functions.<p>
 * <hr>
 * <P>
 * <bold>Copyright</bold> &#169 1995, 1996, 1997
 * <a href="http://www.systemics.com/">Systemics Ltd</a>
 * on behalf of the
 * <a href="http://www.systemics.com/docs/cryptix/">
 * Cryptix Development Team</a>.
 * All rights reserved.
 * @version 1.2, 7/2/1997
 * @author Mike Wynn
 * @author Ian Brown
 */
public class PublicKeyRing extends KeyRing
{
	public PublicKeyRing() {} // Use add()

	/**
	 * Create a new public keyring object from <code>in</code>.
	 */
	public PublicKeyRing( InputStream in )
	throws IOException
	{
		super( in );
	}

	/**
	 * Create a new public keyring object from the file <code>filename</code>.
	 * @deprecated Use PublicKeyRing( File name ) instead
	 */
	public PublicKeyRing( String filename )
	throws FileNotFoundException, IOException
	{
		super( filename );
	}

	/**
	 * Create a new public keyring object from the file <code>filename</code>.
	 */
	public PublicKeyRing( File name )
	throws FileNotFoundException, IOException
	{
		super( name );
	}
	/** Retrieve a key by any of the user names on it.*/
	public PublicKey
	getKey( String username )
	throws IOException
	{
		KeyID toID;
		if ( (toID = (KeyID)( name2keyId.get( username ))) != null )
		{
			PublicKey publicKey = getKey( toID );
			if ( publicKey != null )
				return publicKey;
			// there SHOULD never be a time when you can get the ID but not the key !!
		}
		return getIfBegins( name2keyId.keys(), username );
	}
	
	/** Retrieve a key by its ID.*/
	public PublicKey
	getKey( KeyID keyID )
	throws IOException
	{
		return getPublicKey( keyID );
	}

	/** Retrieve a key by any of the e-mail addresses on it.*/
	public PublicKey
	getKey( EmailAddress address )
	throws IOException
	{
		return getPublicKey( address );
	}

	protected final void
	init( PacketInputStream reader )
	throws IOException
	{
		KeyRingEntry entry;
		int noKeys = 0;
		//System.out.print( "Loading keys" );
		// By default, key entries without trust information will be given undefined trust
		while ( ( entry = reader.readKeyRingEntry( KeyRingTrust.UNDEF ) ) != null )
		{
			if ( ! entry.certificate().expired() ) add( entry );
			//System.out.print( "." );
			noKeys++;
		}
	//	System.out.println( "\n" + noKeys + " keys loaded." );
	}

}

