// This file is currently unlocked (change this line if you lock the file)
//
// Source code version is 2.3.0
//                        (increment this number EVERY time you make a change)
//
// History: (add to this EVERY time you make a change) 
// (insert details of your change immediately below this line)
//
// 2.3.0        16 May 1997     Jill Baker
//      Original Version
//
// If this file is unlocked, or locked by someone other than you, then you
// may download this file for incorporation into your build, however any
// changes you make to your copy of this file will be considered volatile,
// and will not be incorporated into the master build which is maintained
// at this web site.
//
// If you wish to make permanent changes to this file, you must wait until
// it becomes unlocked (if it is not unlocked already), and then lock it
// yourself. Whenever the file is locked by you, you may upload new master
// versions to the master site. Make sure you update the history information
// above. When your changes are complete, you should relinquish the lock to
// make it available for other developers.

/*
 * Copyright (C) 1995, 1996 Systemics Ltd (http://www.systemics.com/)
 * All rights reserved.
 *
 * This library and applications are FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 * as long as the conditions within the COPYRIGHT file are adhered to.
 *
 */

package cryptix.pgp.engine;

import cryptix.io.Lang;
import cryptix.pgp.*;
import cryptix.pgp.engine.toolkit.*;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.PrintWriter;

/**
 * @author Jill Baker
 */
public final class EncryptionEngine extends DecryptionEngine
{
    private byte[] originalData;

    public EncryptionEngine( Toolkit toolkit, byte[] data, String name, char mode )
        throws FormatException, IOException
    {
        super( null );
        originalData = data;
        try
        {
            PacketByteArray pbaData = new PacketByteArray( data );
            // If we're still here then we were given a packet sequence...

            String prompt = Lang.translate( "Is it safe to assume that '[]' is a PGP-format file?" );
            prompt = Lang.escape( prompt, name );

            // Ask the user to confirm our suspicion.
            if (!toolkit.ui.promptYesNo( prompt, false )) throw new FormatException();
            // If we're still here then the user said yes.

            if (toolkit.originalFilename.endsWith( ".pgp" ))
            {
                int len = toolkit.originalFilename.length();
                toolkit.originalFilename = toolkit.originalFilename.substring( 0, len-4 );
            }
            pba = pbaData;
        }
        catch (FormatException e)
        {
            String fileSeparator = System.getProperty( "file.separator" );
            int pos = name.lastIndexOf( fileSeparator );
            if (pos != -1) name = name.substring( pos + fileSeparator.length() );

            ByteArrayOutputStream bout = new ByteArrayOutputStream();
            PacketOutputStream pout = new PacketOutputStream( bout );
            pout.writePacket( new LiteralData( data, name, mode ));
            pout.close();

            pba = new PacketByteArray( bout.toByteArray() );
        }
    }

    public byte[] getOriginalData()
    {
        return originalData;
    }

    /**
     * A real (though very simple) command line PGP public key encryptor!
     */
    public static void main( String[] arg ) throws IOException, DecryptException, FormatException
    {
        // Uncomment the following line to enable debugging.
        Utility.debug = new PrintWriter( System.out, true );

        // Parse the command line
        int n = 0;
        String options = arg[0];
        if (options.startsWith( "-" ))
        {
            n = 1;
            options = options.toLowerCase();
        }
        else
        {
            throw new FormatException( "Options required" );
        }
        String srcFile = arg[1];
        String dstFile = arg.length > 2 ? arg[2] : null;
        String ext = ".pgp";

        // Gather all the details together
        Toolkit toolkit = new Toolkit( srcFile );
        Engine engine = toolkit.getEngine();

        // Process it...
        if (options.indexOf( "s" ) != -1)
        {
            if (options.indexOf( "b" ) != -1)
            {
                ext = ".sig";
                engine = new DetachedSignatureEngine( toolkit, engine );
            }
            else
                engine = new SignatureEngine( toolkit, engine );
        }
        if (!toolkit.config.getProperty( "compress", "on" ).equalsIgnoreCase( "off" ))
            engine = new CompressionEngine( toolkit, engine );
        if (options.indexOf( "e" ) != -1)
            engine = new PublicKeyEncryptionEngine( toolkit, engine );
        if (options.indexOf( "c" ) != -1)
            engine = new ConventionalEncryptionEngine( toolkit, engine );

        // Output the file
        if (options.indexOf( "a" ) != -1)
        {
            engine = new ArmourEngine( toolkit, engine );
            toolkit.setOutputStream( ".asc", (arg.length==2) ? null : dstFile );
            DataOutputStream dos = new DataOutputStream( toolkit.outputStream );
            dos.writeBytes( engine.toString() );
            dos.close();
        }
        else
        {
            toolkit.setOutputStream( ext, (arg.length==2) ? null : dstFile );
            toolkit.outputStream.write( engine.toPacketByteArray().toByteArray() );
        }

        // All done. Tell the user.
        toolkit.ui.notice( Lang.translate( "Output file" ) + ": '" + toolkit.destFilename + "'" );
    }


}
