// This file is currently unlocked (change this line if you lock the file)
//
// Source code version is 2.3.0
//                        (increment this number EVERY time you make a change)
//
// History: (add to this EVERY time you make a change) 
// (insert details of your change immediately below this line)
//
// 2.3.0        16 May 1997     Jill Baker
//      Original Version
//
// If this file is unlocked, or locked by someone other than you, then you
// may download this file for incorporation into your build, however any
// changes you make to your copy of this file will be considered volatile,
// and will not be incorporated into the master build which is maintained
// at this web site.
//
// If you wish to make permanent changes to this file, you must wait until
// it becomes unlocked (if it is not unlocked already), and then lock it
// yourself. Whenever the file is locked by you, you may upload new master
// versions to the master site. Make sure you update the history information
// above. When your changes are complete, you should relinquish the lock to
// make it available for other developers.

/*
 * Copyright (C) 1995, 1996 Systemics Ltd (http://www.systemics.com/)
 * All rights reserved.
 *
 * This library and applications are FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 * as long as the conditions within the COPYRIGHT file are adhered to.
 *
 */

package cryptix.pgp.engine;

import cryptix.io.Lang;
import cryptix.math.RandomStream;
import cryptix.pgp.*;
import cryptix.pgp.engine.toolkit.*;
import cryptix.security.MD5;
import cryptix.security.rsa.SecretKey;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintWriter;

/**
 * @author Jill Baker
 */
public class SignatureEngine extends Engine implements SignatureConstants
{
    protected Signature signature;
    protected PacketByteArray pba;

    /**
     * Constuctor which merely puts all the elements together,
     * assuming they are correct.
     */
    /*package-visibility*/ SignatureEngine( Signature signature, PacketByteArray pba )
    {
        this.signature = signature;
        this.pba = pba;
    }

    /**
     * Constructor which signs an existing engine.
     * @param toolkit toolkit defining user preferences
     * @engine Engine object to sign
     * @return Engine, now signed
     */
    public SignatureEngine( Toolkit toolkit, Engine engine ) throws IOException, DecryptException
    {
        this( toolkit, engine, toolkit.getSecretKey() );
    }

    /**
     * Constructor which signs an existing engine.
     * @param toolkit toolkit defining user preferences
     * @param engine Engine object to sign
     * @param key SecretKey with which to sign
     * @return Engine, now signed
     */
    public SignatureEngine( Toolkit toolkit, Engine engine, SecretKey key ) throws IOException, DecryptException
    {
        if (key == null) throw new DecryptException( "No Secret Key" );
        pba = engine.toPacketByteArray();

        MD5 md = new MD5();
        if (engine instanceof EncryptionEngine)
            md.add( ((EncryptionEngine) engine).getOriginalData() );
        else
            md.add( pba.toByteArray() );
        signature = new Signature( key, md, toolkit.randomStream );
    }

    public DecryptStep decryptStep( Toolkit toolkit ) throws DecryptException, IOException
    {
        toolkit.ui.status( Lang.translate( "Checking signature" ));
        sigCheck:
        {
            byte[] plaintext;
            KeyGlob keyRings = toolkit.keyRings;
            if (keyRings == null) break sigCheck;
            cryptix.security.rsa.PublicKey key = null;
            KeyID keyId = signature.getKeyId();
            try { key = keyRings.getKey( keyId ); } catch (IOException e) {}
            if (key == null) break sigCheck;

            int trust = keyRings.getKeyTrust( keyId, true );

            if (signature.verify( key, toolkit.plaintextGetter.get() ))
            {
                toolkit.ui.status( Lang.translate( "Verify succeeded" ));
                switch( trust )
                {
                    case 0:  toolkit.sigCheckValue = GOOD_SIG_NEVER_TRUST;     break;
                    case 2:  toolkit.sigCheckValue = GOOD_SIG_SOMETIMES_TRUST; break;
                    case 3:  toolkit.sigCheckValue = GOOD_SIG_ALWAYS_TRUST;    break;
                    default: toolkit.sigCheckValue = GOOD_SIG_UNKNOWN_TRUST;   break;
                }
            }
            else
            {
                toolkit.ui.warning( Lang.translate( "Verify failed" ));
                switch( trust )
                {
                    case 0:  toolkit.sigCheckValue = BAD_SIG_NEVER_TRUST;     break;
                    case 2:  toolkit.sigCheckValue = BAD_SIG_SOMETIMES_TRUST; break;
                    case 3:  toolkit.sigCheckValue = BAD_SIG_ALWAYS_TRUST;    break;
                    default: toolkit.sigCheckValue = BAD_SIG_UNKNOWN_TRUST;   break;
                }
            }
            toolkit.plaintextGetter = null;
        }
        return new DecryptStep( pba );
    }

    public PacketByteArray toPacketByteArray() throws IOException
    {
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        DataOutputStream dout = new DataOutputStream( out );
        PacketOutputStream pout = new PacketOutputStream( dout );
        pout.writePacket( signature );
        pout.close();
        dout.close();
        out.write( pba.toByteArray() );
        return new PacketByteArray( out.toByteArray() );
    }

    /**
     * A real (though very simple) command line PGP signer!
     */
    public static void main( String[] arg ) throws IOException, DecryptException, FormatException
    {
        // Uncomment the following line to enable debugging.
        Utility.debug = new PrintWriter( System.out, true );

        // Gather all the details together
        Toolkit toolkit = new Toolkit( arg[0] );
        Engine engine = toolkit.getEngine();

        // Sign it...
        try { engine = new SignatureEngine( toolkit, engine ); }
        catch (DecryptException e) { return; }

        // Output the file
        toolkit.setOutputStream( ".pgp", (arg.length==1) ? null : arg[1] );
        toolkit.outputStream.write( engine.toPacketByteArray().toByteArray() );

        // All done. Tell the user.
        toolkit.ui.notice( Lang.translate( "Signed file" ) + ": '" + toolkit.destFilename + "'" );
    }
}
