package enigma;

import java.awt.*;

/**
 * A GUI dialog to get the user's passphrase, which is hidden by ****<p>
 * Use <code>getPassphrase()</code> to retrieve it.
 * @author Ian Brown
 * @version 0.9
 */ 
public class PassphraseDialog extends Dialog
{
    String phrase = "";
    Button OKButton = new Button("Ok");
    Button quitButton = new Button("Quit");
    TextField passphrase = new TextField(30);

	/**
	 * Create a new dialog box with title asking for the passphrase for id.<p>
	 * Note that you *need* a parent window - create a dummy hidden one if
	 * necessary.
	 * @param parent The window that is creating this dialog
	 * @param title The title of the dialog
	 * @param id The user ID of the key whose passphrase you require
	 */
	public PassphraseDialog( Frame parent, String title, String id )
	{
		super( parent, title, true );
		setBackground(Color.lightGray);
		setLayout( new BorderLayout() );
		Panel top = new Panel();
		Panel middle = new Panel();
		Panel bottom = new Panel();
		top.setLayout( new BorderLayout() );
		top.add( "North", new Label("Please enter the passphrase for" ));
		top.add( "Center", new Label( id ));
		middle.setLayout(new BorderLayout());
		passphrase.setEchoCharacter( '*' );
		middle.add("North", passphrase);
		bottom.setLayout(new FlowLayout());
		bottom.add(OKButton);
		bottom.add(quitButton);
		add("North", top);
		add("Center", middle);
		add("South", bottom);
        	resize( 200, 140 );
		setResizable( false );
		passphrase.requestFocus();
		show();     
	}

    public synchronized boolean handleEvent(Event e)
    {
        if (e.id == Event.WINDOW_DESTROY)
        {
            System.exit( 1 );
            return true;
        }

        else if (e.id == Event.ACTION_EVENT && e.target == quitButton)
        {
            System.exit( 1 );
	    return true;
        }

        else if (e.id == Event.ACTION_EVENT && e.target == OKButton)
        {
    	    phrase = passphrase.getText();
            this.hide();
    	    return true;
        }

    	else return false;
    }

    public String getPassphrase( )
    {
        return phrase;
    }
}
